window.Gleam.initSearch([{"doc":"mysig","title":"mysig","content":"# mysig\n\nCosy toolkit to make you feel at home.\n\n[![Package Version](https://img.shields.io/hexpm/v/mysig)](https://hex.pm/packages/mysig)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/mysig/)\n\n```sh\ngleam add mysig\n```\n\nThe project include several tools that I have for making it easier to build apps with Gleam (and Lustre).\n*For the moment the project assumes lustre but that constraint should relax over time.*\n\nThe project acts as a bundler for Gleam web applications. It tries to.\n\n- Avoid compiler magic\n- Create the same out put in dev as in prod.\n- Lazily build content in dev to remain fast\n\n## Concepts\n\n### Routes\n\nRoutes are the tree of endpoints that your application has.\n\n### Assets\n\nThe main bundler is defined in the asset modules.\n- `mysig/asset` allows you to declare what assets a component/page has\n- `mysig/server` will collect all the assets referenced from the file system\n- `mysig/client` will replace all asset lookup with fingerprinted values created by the server\n\n\n\n### Other approaches to bundling\n\nMysig aims to be simple and **avoid compiler magic**. For example:\n\n```js\nimport asset from &quot;./cat.jpg&quot;\n```\n\nThis API is ergonomic results in build tooling that needs to understand the JavaScript syntax.\nI don&#39;t want to build another Gleam compiler.\n\nMy current goal is to make a data structure that represents the content of a web application efficiently.\n\nThe datastructure should be lazy so that in dev you can build quickly on demand, and for production walk the whole data structure eagerly.\nIn this way **Development should product the same artifacts as Production**.\n\n#### Gathering references at runtime only works for static site generation.\n\nFor example a view function.\n```rs\npub fn render(bundle, state) {\n  case state {\n    True -&gt; {\n      use logo_bits &lt;- t.do(t.read(&quot;logo.png&quot;))\n      use logo_src &lt;- t.do(asset.png(&quot;logo&quot;, logo_bits))\n      // build wont find this if state for SSR is false\n    }\n    False\n  }\n}\n```\n\nTherefore a runtime API needs to require building of assets to happen before any state control flow\n```rs\npub fn view() {\n  use asset &lt;- load()\n  fn(state) {\n    // render html\n  }\n}\n```\nReturning the render function from a Build task ensures that all assets are referenced before any control flow.\nHowever the API gets more verbose.\n\nA new data structure representing if,loop,await in templates would allow walking the whole tree.\n\n#### Asset lookup and build tool\n\nIf there was some way to specify all the assets you could have a build task\n\nStart with a toml declaration and run `gleam run mysig/manifest/gen`\nWorks well with autocomplete in the editor. Doesn&#39;t usage discovery of assets, it also requires a total rebuild for any asset changes.\n\n- Images don&#39;t update very often\n- A task to create assets allows them to stay in git.\n\n#### Investigating the lustre content\n\nWalking the lustre tree and finding all source attributes would have the same problems as runtime generation.\ni.e. if rendered on the server with a specific state there would be no way to discover assets that where no used with that state.\n\n#### Find and replace in bundle\n\nOnce bundling we can find and replace all references using a magic string &quot;__asset__/foo.png?w=20&quot;\n\n### Older tools in this repo\n\nThese are still include as they have some nice approaches but I think they are probably superceeded by the main approach above\n\n- open_graph, helpful builders\n- local, services static set of files\n\n## Live reloading for local dev\n\nIn the `mysig/local` module, works with watching midas tasks.\n\nSee https://vimeo.com/1030747035\n\n### Other Gleam tools\n\n##### Pensive https://github.com/Pevensie/pevensie\nA backend framework focusing on Auth and DB integration Mysig is instead a metaframework for front end first apps.\n\n##### Arctic https://github.com/RyanBrewer317/arctic\nFocuses on pulling content from the file system and composing parsers.\nI want Mysig to focus on working with already structured data and ignoring the file system as much as possible.\n\n##### Meadow https://github.com/JoelVerm/meadow\nNot very active focuses on adding server signals to a solidjs client. I like the sound of this approach.\n\n## Credit\n\nCreated for the [EYG](https://eyg.run/) homepage. EYG is a new integration focused programming language.\n\nIf startup is lazy then we just restart on any code change This would need file watching to trigger the restart\nThe client can inject to listen to a websocket and if it looses connection or reload the page when a new connection arrises\nGleam build is fast\n\nEntry points can be listed when building\n\nString replace involves parsing eventually. magic string is possible\n\nGen is not lazy BUT we cannot see all things unless walk the code\nbuild using a mysig/manifest/gen\nlustre dev tools finds main from toml name https://github.com/lustre-labs/dev-tools/blob/main/src/lustre_dev_tools/project.gleam#L86","url":"index.html"},{"doc":"mysig/asset","title":"Asset","content":"pub type Asset {\n  Ref(String)\n}\n\nRef(String)\n\n\nSynonyms:\nmysig/asset.Asset\nmysig/asset Asset","url":"mysig/asset.html#Asset"},{"doc":"mysig/asset","title":"Effect","content":"pub type Effect(a) {\n  Done(a)\n  Abort(Snag)\n  Load(\n    file: String,\n    resume: fn(Result(Asset, Snag)) -&gt; Effect(a),\n  )\n  Bundle(\n    module: String,\n    function: String,\n    resume: fn(Result(Asset, Snag)) -&gt; Effect(a),\n  )\n}\n\nDone(a)\n\n\nAbort(Snag)\n\n\nLoad(file: String, resume: fn(Result(Asset, Snag)) -&gt; Effect(a))\n\n\nBundle(\n  module: String,\n  function: String,\n  resume: fn(Result(Asset, Snag)) -&gt; Effect(a),\n)\n\n\nSynonyms:\nmysig/asset.Effect\nmysig/asset Effect","url":"mysig/asset.html#Effect"},{"doc":"mysig/asset","title":"bundle","content":"pub fn bundle(module: String, function: String) -&gt; Effect(Asset)\n\nSynonyms:\nmysig/asset.bundle\nmysig/asset bundle","url":"mysig/asset.html#bundle"},{"doc":"mysig/asset","title":"do","content":"pub fn do(eff: Effect(a), then: fn(a) -&gt; Effect(b)) -&gt; Effect(b)\n\nSynonyms:\nmysig/asset.do\nmysig/asset do","url":"mysig/asset.html#do"},{"doc":"mysig/asset","title":"done","content":"pub fn done(x: a) -&gt; Effect(a)\n\nSynonyms:\nmysig/asset.done\nmysig/asset done","url":"mysig/asset.html#done"},{"doc":"mysig/asset","title":"load","content":"pub fn load(file: String) -&gt; Effect(Asset)\n\nSynonyms:\nmysig/asset.load\nmysig/asset load","url":"mysig/asset.html#load"},{"doc":"mysig/asset","title":"src","content":"pub fn src(asset: Asset) -&gt; String\n\nSynonyms:\nmysig/asset.src\nmysig/asset src","url":"mysig/asset.html#src"},{"doc":"mysig/asset","title":"mysig/asset","content":"","url":"mysig/asset.html"},{"doc":"mysig/asset/client","title":"load_manifest","content":"pub fn load_manifest(eff: Effect(a)) -&gt; Result(a, Snag)\n\nSynonyms:\nmysig/asset/client.load_manifest\nmysig/asset/client load_manifest","url":"mysig/asset/client.html#load_manifest"},{"doc":"mysig/asset/client","title":"mysig/asset/client","content":"","url":"mysig/asset/client.html"},{"doc":"mysig/asset/server","title":"build_manifest","content":"pub fn build_manifest(\n  eff: Effect(a),\n  assets: Dict(String, #(String, String, BitArray)),\n) -&gt; Promise(\n  Result(#(a, Dict(String, #(String, String, BitArray))), Snag),\n)\n\nSynonyms:\nmysig/asset/server.build_manifest\nmysig/asset/server build_manifest","url":"mysig/asset/server.html#build_manifest"},{"doc":"mysig/asset/server","title":"mysig/asset/server","content":"","url":"mysig/asset/server.html"},{"doc":"mysig/route","title":"Endpoint","content":"pub type Endpoint {\n  Page(asset.Effect(String))\n  Static(content: BitArray)\n}\n\nPage(asset.Effect(String))\n\n\nStatic(content: BitArray)\n\n\nSynonyms:\nmysig/route.Endpoint\nmysig/route Endpoint","url":"mysig/route.html#Endpoint"},{"doc":"mysig/route","title":"Route","content":"pub type Route {\n  Route(index: Endpoint, items: List(#(String, Route)))\n}\n\nRoute(index: Endpoint, items: List(#(String, Route)))\n\n\nSynonyms:\nmysig/route.Route\nmysig/route Route","url":"mysig/route.html#Route"},{"doc":"mysig/route","title":"match","content":"pub fn match(\n  segments: List(String),\n  route: Route,\n) -&gt; Result(Endpoint, Nil)\n\nSynonyms:\nmysig/route.match\nmysig/route match","url":"mysig/route.html#match"},{"doc":"mysig/route","title":"mysig/route","content":"","url":"mysig/route.html"},{"doc":"mysig/build","title":"to_files","content":"pub fn to_files(\n  route: Route,\n) -&gt; Promise(Result(List(#(String, BitArray)), Snag))\n\nSynonyms:\nmysig/build.to_files\nmysig/build to_files","url":"mysig/build.html#to_files"},{"doc":"mysig/build","title":"mysig/build","content":"","url":"mysig/build.html"},{"doc":"mysig/dev","title":"serve","content":"pub fn serve(route: Route) -&gt; Nil\n\nSynonyms:\nmysig/dev.serve\nmysig/dev serve","url":"mysig/dev.html#serve"},{"doc":"mysig/dev","title":"mysig/dev","content":"","url":"mysig/dev.html"},{"doc":"mysig/html","title":"tailwind_2_2_11","content":"pub const tailwind_2_2_11: String\n\nSynonyms:\nmysig/html.tailwind_2_2_11\nmysig/html tailwind_2_2_11","url":"mysig/html.html#tailwind_2_2_11"},{"doc":"mysig/html","title":"common_head_tags","content":"pub fn common_head_tags() -&gt; List(Element(a))\n\nSynonyms:\nmysig/html.common_head_tags\nmysig/html common_head_tags","url":"mysig/html.html#common_head_tags"},{"doc":"mysig/html","title":"doc","content":"pub fn doc(\n  head: List(Element(a)),\n  body: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nmysig/html.doc\nmysig/html doc","url":"mysig/html.html#doc"},{"doc":"mysig/html","title":"empty_lustre","content":"pub fn empty_lustre() -&gt; Element(a)\n\nSynonyms:\nmysig/html.empty_lustre\nmysig/html empty_lustre","url":"mysig/html.html#empty_lustre"},{"doc":"mysig/html","title":"plausible","content":"pub fn plausible(domain: String) -&gt; Element(a)\n\nSynonyms:\nmysig/html.plausible\nmysig/html plausible","url":"mysig/html.html#plausible"},{"doc":"mysig/html","title":"stylesheet","content":"pub fn stylesheet(reference: String) -&gt; Element(a)\n\nSynonyms:\nmysig/html.stylesheet\nmysig/html stylesheet","url":"mysig/html.html#stylesheet"},{"doc":"mysig/html","title":"mysig/html","content":"","url":"mysig/html.html"},{"doc":"mysig/local","title":"handler","content":"pub fn handler(\n  content: List(#(String, BitArray)),\n) -&gt; Effect(fn(Request(a)) -&gt; Response(BitArray))\n\nSynonyms:\nmysig/local.handler\nmysig/local handler","url":"mysig/local.html#handler"},{"doc":"mysig/local","title":"inject_live_reload","content":"pub fn inject_live_reload(\n  hash: String,\n  bytes: BitArray,\n) -&gt; BitArray\n\nSynonyms:\nmysig/local.inject_live_reload\nmysig/local inject_live_reload","url":"mysig/local.html#inject_live_reload"},{"doc":"mysig/local","title":"serve","content":"pub fn serve(\n  port: Option(Int),\n  content: List(#(String, BitArray)),\n) -&gt; Effect(Nil)\n\nSynonyms:\nmysig/local.serve\nmysig/local serve","url":"mysig/local.html#serve"},{"doc":"mysig/local","title":"mysig/local","content":"","url":"mysig/local.html"},{"doc":"mysig/open_graph","title":"description","content":"pub fn description(content: String) -&gt; Element(a)\n\nSynonyms:\nmysig/open_graph.description\nmysig/open_graph description","url":"mysig/open_graph.html#description"},{"doc":"mysig/open_graph","title":"image","content":"pub fn image(content: Uri) -&gt; Element(a)\n\nSynonyms:\nmysig/open_graph.image\nmysig/open_graph image","url":"mysig/open_graph.html#image"},{"doc":"mysig/open_graph","title":"image_alt","content":"pub fn image_alt(content: String) -&gt; Element(a)\n\nSynonyms:\nmysig/open_graph.image_alt\nmysig/open_graph image_alt","url":"mysig/open_graph.html#image_alt"},{"doc":"mysig/open_graph","title":"image_height","content":"pub fn image_height(content: Int) -&gt; Element(a)\n\nSynonyms:\nmysig/open_graph.image_height\nmysig/open_graph image_height","url":"mysig/open_graph.html#image_height"},{"doc":"mysig/open_graph","title":"image_type","content":"pub fn image_type(content: String) -&gt; Element(a)\n\nSynonyms:\nmysig/open_graph.image_type\nmysig/open_graph image_type","url":"mysig/open_graph.html#image_type"},{"doc":"mysig/open_graph","title":"image_width","content":"pub fn image_width(content: Int) -&gt; Element(a)\n\nSynonyms:\nmysig/open_graph.image_width\nmysig/open_graph image_width","url":"mysig/open_graph.html#image_width"},{"doc":"mysig/open_graph","title":"site_name","content":"pub fn site_name(content: String) -&gt; Element(a)\n\nSynonyms:\nmysig/open_graph.site_name\nmysig/open_graph site_name","url":"mysig/open_graph.html#site_name"},{"doc":"mysig/open_graph","title":"title","content":"pub fn title(content: String) -&gt; Element(a)\n\nSynonyms:\nmysig/open_graph.title\nmysig/open_graph title","url":"mysig/open_graph.html#title"},{"doc":"mysig/open_graph","title":"url","content":"pub fn url(content: Uri) -&gt; Element(a)\n\nSynonyms:\nmysig/open_graph.url\nmysig/open_graph url","url":"mysig/open_graph.html#url"},{"doc":"mysig/open_graph","title":"website","content":"pub fn website() -&gt; Element(a)\n\nSynonyms:\nmysig/open_graph.website\nmysig/open_graph website","url":"mysig/open_graph.html#website"},{"doc":"mysig/open_graph","title":"mysig/open_graph","content":"","url":"mysig/open_graph.html"},{"doc":"mysig/preview","title":"png","content":"pub const png: String\n\nSynonyms:\nmysig/preview.png\nmysig/preview png","url":"mysig/preview.html#png"},{"doc":"mysig/preview","title":"homepage","content":"pub fn homepage(\n  title title: String,\n  description description: String,\n  canonical canonical: Uri,\n) -&gt; List(Element(a))\n\nSynonyms:\nmysig/preview.homepage\nmysig/preview homepage","url":"mysig/preview.html#homepage"},{"doc":"mysig/preview","title":"image","content":"pub fn image(\n  url url: Uri,\n  type_ type_: String,\n  width width: Int,\n  height height: Int,\n  alt alt: String,\n) -&gt; List(Element(a))\n\nSynonyms:\nmysig/preview.image\nmysig/preview image","url":"mysig/preview.html#image"},{"doc":"mysig/preview","title":"optimum_image","content":"pub fn optimum_image(\n  url url: Uri,\n  type_ type_: String,\n  alt alt: String,\n) -&gt; List(Element(a))\n\nSynonyms:\nmysig/preview.optimum_image\nmysig/preview optimum_image","url":"mysig/preview.html#optimum_image"},{"doc":"mysig/preview","title":"page","content":"pub fn page(\n  site site: String,\n  title title: String,\n  description description: String,\n  canonical canonical: Uri,\n) -&gt; List(Element(a))\n\nSynonyms:\nmysig/preview.page\nmysig/preview page","url":"mysig/preview.html#page"},{"doc":"mysig/preview","title":"mysig/preview","content":"","url":"mysig/preview.html"}]);